<?php

require("locals.php");

/*
 * open_weatherstation
 *
 * Input:   serial port where station is connected
 * 
 * Returns: nothing
 *
*/
function open_weatherstation( $port )
{
	ser_open( $port, 2400, 8, "None", 1, "None" );
	ser_setDTR( false );
	ser_setRTS( true );
}

/*
 * close_weatherstation
 *
 * Input:   nothing
 * 
 * Returns: nothing
 *
*/
function close_weatherstation()
{
	ser_close();
}

/*
 * reset_06 WS2300 by sending command 06 (PHP version) 
 * 
 * Input:   nothing
 *           
 * Returns: nothing
 *
 *
 */
function reset_06()
{
	$command = chr(0x06);
	$ret = false;
	global $maxtrials;
	global $rw_timeout;
	
	for ($i = 0; $i < $maxtrials; $i++)
	{
		write_device($command);

		read_device(&$answer);
		
		if (strstr($answer, chr(0x02)))
		{
			$ret = true;
			break;
		}

		usleep($rw_timeout * $i);
	}
	return $ret;
}

/*
 * read_safe Read data, retry until success or maxretries
 * Reads data from the WS2300 based on a given address,
 * number of data read, and a an already open serial port
 * Uses the read_data function and has same interface
 *
 * Inputs:  address (interger - 16 bit)
 *          number - number of bytes to read, max value 15
 *
 * Output:  readdata - received data
 * 
 * Returns: number of bytes read, -1 if failed
 *
 */
function read_safe($address, &$readdata, $number)
{
	global $maxtrials;

	for ($j = 0; $j < $maxtrials; $j++)
	{
		reset_06();
		flush_serial();
		// Read the data. If expected number of bytes read break out of loop.
		$got = read_data($address, &$readdata, $number);
		if ($got == $number)
		{
			break;
		}
	}

	// If we have tried $maxtrials times to read we expect not to
	// have valid data
	if ($j == $maxtrials)
	{
		return -1;
	}

	return $number;
}

/*
 * data_checksum calculates the checksum for the data bytes received
 * from the WS2300
 *
 * Input:   array
 * 
 * Returns: calculated checksum as unsigned char
 *
 */
function data_checksum($data, $number)
{
	$checksum = 0;

	for ($i = 1; $i < $number+1; $i++)
	{
		$checksum = $checksum + $data[$i];
	}

	$checksum &= 0xFF;

	return $checksum;
}

/*
 * numberof_encoder converts the number of bytes we want to read
 * to the form needed by the WS-2300 when sending commands.
 *
 * Input:   number interger, max value 15
 * 
 * Returns: unsigned char which is the coded number of bytes
 *
 */
function numberof_encoder($number)
{
	$coded_number = 0xC2 + $number * 4;
	if ($coded_number > 0xfe)
		$coded_number = 0xfe;

	return chr($coded_number);
}

 /*
 * address_encoder converts an 16 bit address to the form needed
 * by the WS-2300 when sending commands.
 *
 * Input:   address_in (interger - 16 bit)
 * 
 * Output:  address_out - array
 * 
 * Returns: Nothing.
 *
 ********************************************************************/
function address_encoder($address_in)
{
	$adrbytes = 4;
	$nibble;
	$address_out = array();

	for ($i = 0; $i < $adrbytes; $i++)
	{
		$nibble = ($address_in >> (4 * (3 - $i))) & 0x0F;
		$address_out[$i] = chr(0x82 + ($nibble * 4));
	}
	return $address_out;
}

/*
 * read_data reads data from the WS2300 based on a given address,
 * number of data read
 *
 * Inputs:  address (interger - 16 bit)
 *          number - number of bytes to read, max value 15
 *
 * Output:  readdata - received data
 * 
 * Returns: number of bytes read, -1 if failed
 *
 */
function read_data($address, &$readdata, $number)
{
	global $rw_timeout;

	// First 4 bytes are populated with converted address range 0000-13B0
	$commanddata = address_encoder($address);
	// Last populate the 5th byte with the converted number of bytes
	$p = numberof_encoder($number);

	$commanddata[4] = $p; 

	for ($i = 0; $i < 4; $i++)
	{
		ser_writebyte(ord($commanddata[$i]));
		usleep($rw_timeout);
		ser_readbyte();
	}

	ser_writebyte(ord($commanddata[4]));
	usleep($rw_timeout);
	ser_readbyte();

	//Read the data bytes
	for ($i = 0; $i < $number+2; $i++)
	{
		usleep($rw_timeout);
		$readdata[$i] = ser_readbyte();
	}

	if ($readdata[$number + 1] != data_checksum($readdata, $number))
		return -1;
		
	return $number;
}

/*
 * write_safe Write data, only for compatibilty
 *
 * Returns: nothing
 *
*/
function write_safe($writedata)
{
	reset_06();
	write_data($writedata);
	
}

/*
 * read_device PHP emulation
 * Reads data from the serial device
 *
 * Inputs:  nothing
 *
 * Outputs: received data
 * Returns: number of bytes read
*/
function read_device(&$received)
{
	$received = ser_read(16);
	$count = strlen($received);
	
	return $count;
}

/*
 * write_device PHP emulation 
 * Writes data to the serial device
 *
 * Inputs:  data - data to be sent
 *
 * Returns: nothing
 *
*/
function write_device($data)
{
	ser_write($data);
}

function flush_serial()
{
usleep(30000);	
while (strlen(ser_read(16)) > 0) 
{
	usleep(30000);
	// this simulates a flush
}
}

?>